#include <stdio.h>

    /*
     * Zeigertyp fuer Funktionen der folgenden Art:
     *     double FunktionsName(double param);
     */
typedef double (*Function)(double);

    /*
     * Numerische Integration der Funktion h nach der Trapezregel
     * h: zu integrierende Funktion
     * [x0,x1]: Integrationsbereich
     * n: Anzahl der Intervalle im Integrationsbereich
     */
double integrate(Function h, double x0, double x1, int n) {
    double delta = (x1 - x0) / n;
    double sum = 0.0;
    double x;
    int i;
    for (i = 0, x = x0; i <= n; i++, x += delta)
	sum += (*h)(x);
    return (sum - ((*h)(x0) + (*h)(x1)) / 2.0) * delta;
}

/*
 * zwei Beispiel-Funktionen
 */
double f(double x) {
    return x * x;
}
double g(double x) {
    return 2.0 * x * x - 3.0 * x + 1.0;
}

int main() {
    puts("Numerische Integration:");
    printf("f: %lf\n", integrate(&f, 0, 1, 100));
    printf("g: %lf\n", integrate(&g, 0, 1, 100));
    
    return 0;
}
